
/*
      _______                          .__ .__   .__   .__                 
      \      \    ____   __ __ _______ |__||  |  |  |  |__|  ____    ____  
      /   |   \ _/ __ \ |  |  \\_  __ \|  ||  |  |  |  |  | /  _ \  /    \ 
     /    |    \\  ___/ |  |  / |  | \/|  ||  |__|  |__|  |(  <_> )|   |  \
     \____|__  / \___  >|____/  |__|   |__||____/|____/|__| \____/ |___|  /
    =========\/======\/=================================================\/==
  v0.01 19/FEB/2008   Copyright 2007-2008 Scott D. Yelich SOME RIGHTS RESERVED
  ,-*~'`^`'~*-,._.,-*~'`^`'~*-,._.,-*~'`^`'~*-,._.-*~'`^`'~*-,._.,-*~'`^`'~*-,


  LICENSE:  Creative Commons Attribution 3.0 License.
  SEE:      http://creativecommons.org/licenses/by/3.0/


  Tue Feb 19 23:05:26 EST 2008, v0.01 sdy

*/

/*
    This is MailSend.cpp -- part of MailSend:

    As the name suggestion -- this application send mail.  It was
    decided that the method to send mail via the SMTP protocol via
    a TCP socket to a MTA's SMTP port.

*/

#include "MailSend.h"

MailSend::MailSend() :
  local_port(3999),
  smtp_name("localhost"),
  helo_name(smtp_name),
  utilities(),
  socket()
{
  srand(time(0));
}

MailSend::~MailSend()
{
}

std::string
MailSend::server (std::string const & server_name)
{
  if (!server_name.empty()) {
    smtp_name = server_name;
  }
  return smtp_name;
}

std::string
MailSend::domain (std::string const & helo_domain)
{
  if (!helo_domain.empty()) {
    helo_name = helo_domain;
  }
  return helo_name;
}

void
MailSend::create_socket()
{
  try {
    socket.create();
    socket.reuseport();
    local_port =  1024+ (int)(30000 * (rand() / (RAND_MAX + 1.0)));
    socket.bind(local_port);
    socket.connect(smtp_name, 25); // FIX
  } catch (TypedException<Socket::Errors> e) {
    std::cerr << "MS::create_socket ERROR code(" << e.code() << ") e.what(" << e.what() << ")" << std::endl;
  }
}

//  WARNING:  THIS MODIFIES THE STRING BEING PASSED IN!  YUK!
int
MailSend::send_line(std::string & line)
{
  int rc = 0;
  rc = socket.send(line);
  if (rc > 0) {
//std::cout << "--> " << line.substr(0, rc);
    line.erase(0, rc+1);
  }
  return rc;
}

std::string
MailSend::get_line()
{
  std::string::size_type nloc = std::string::npos;
  int x;
  std::string s("");
  do { 
    x = socket.recv();
    s = socket.input();
    if (s.size() == 0) {
      throw TypedException<MailSend::Errors>(MailSend::GetBytes, "Get: 0 bytes?");
      // FIX?
      return s;
    }
    nloc = s.find("\n", 0); // painter
  } while (std::string::npos == nloc);
  socket.clearinput(nloc);
  s.erase(nloc,s.size());
  return s;
}

void
MailSend::parse_smtp_response(std::string const & smtp_string)
{
  std::string::size_type nloc = std::string::npos;
//std::cout << "<-- " << smtp_string;
  nloc = smtp_string.find(" ", 0);
  if (std::string::npos != nloc) {
    smtp_response_code = utilities.stringToInt(smtp_string.substr(0, nloc));
    smtp_response = smtp_string;
    smtp_response_string = smtp_string.substr(nloc+1, smtp_string.size()-nloc-3);
  } else {
    smtp_response_code = -1;
    smtp_response = smtp_string;
    smtp_response_string = smtp_string;
  }
}

void
MailSend::send(std::string const & domain, std::string const & from, std::string const & to, std::string const & subject, std::string const & body)
{
  mail.set(from, to, subject, body);
  return send();
}

void
MailSend::send()
{
  create_socket();
  get_initial();
  do_helo();
  do_mail_from();
  do_rcpt_to();
  do_data();
}


void
MailSend::get_initial()
{
  int x = socket.recv(); // blocking
  std::string s = socket.input();
  parse_smtp_response(s);
  if (220 != smtp_response_code) {
    throw TypedException<MailSend::Errors>(MailSend::Init, "Initial message, expecting 220, got: " + utilities.intToString(smtp_response_code) + " " + smtp_response_string);
  } 
}

void
MailSend::do_helo()
{
  oss.str("");
  oss.clear();
  oss << "helo " << helo_name << "\n";
  std::string s = oss.str();
  socket.clearinput();
  int x = send_line(s);  //  check x = len oss.str().size()?
  x = socket.recv(); // blocking
  s = socket.input();
  parse_smtp_response(s);
  if (250 != smtp_response_code) {
    throw TypedException<MailSend::Errors>(MailSend::Helo, "Error after 'helo', expecting 250, got: " + utilities.intToString(smtp_response_code) + " " + smtp_response_string);
  } 
}

void
MailSend::do_mail_from()
{
  oss.str("");
  oss.clear();
  oss << "mail from: <" << mail.from() << ">\n";
  std::string s = oss.str();
  socket.clearinput();
  int x = send_line(s);  //  check x = len oss.str().size()?
  x = socket.recv(); // blocking
  s = socket.input();
  parse_smtp_response(s);
  if (250 != smtp_response_code) {
    throw TypedException<MailSend::Errors>(MailSend::MailFrom, "Error after 'mail from:', expecting 250, got: " + utilities.intToString(smtp_response_code) + " " + smtp_response_string);
  } 
}

void
MailSend::do_rcpt_to()
{
  oss.str("");
  oss.clear();
  oss << "rcpt to: <" << mail.to() << ">\n";
  std::string s = oss.str();
  socket.clearinput();
  int x = send_line(s);  //  check x = len oss.str().size()?
  x = socket.recv(); // blocking
  s = socket.input();
  parse_smtp_response(s);
  if (250 != smtp_response_code) {
    throw TypedException<MailSend::Errors>(MailSend::RcptTo, "Error after 'rcpt to:', expecting 250, got: " + utilities.intToString(smtp_response_code) + " " + smtp_response_string);
  } 
}

void
MailSend::do_data()
{
  oss.str("");
  oss.clear();
  oss << "data\n";
  std::string s = oss.str();
  socket.clearinput();
  int x = send_line(s);  //  check x = len oss.str().size()?
  x = socket.recv(); // blocking
  s = socket.input();
  parse_smtp_response(s);
  if (354 != smtp_response_code) {
    throw TypedException<MailSend::Errors>(MailSend::Data, "Error after 'data', expecting 354, got: " + utilities.intToString(smtp_response_code) + " " + smtp_response_string);
  } 

  oss.str("");
  oss.clear();
  oss << "To: " << mail.from() << std::endl;
  oss << "Subject: " << mail.subject() << std::endl;
  oss << std::endl;
  oss << mail.body() << std::endl;
  s = oss.str();
  socket.clearinput();
  x = send_line(s);  //  check x = len oss.str().size()? FIX?

  oss.str("");
  oss.clear();
  oss << "\n.\n";
  s = oss.str();
  socket.clearinput();
  x = send_line(s);  //  check x = len oss.str().size()?

  x = socket.recv(); // blocking
  s = socket.input();

  parse_smtp_response(s);
  if (250 != smtp_response_code) {
    throw TypedException<MailSend::Errors>(MailSend::Dot, "Error after '.', expecting 250, got: " + utilities.intToString(smtp_response_code) + " " + smtp_response_string);
  } 

  socket.disconnect();
}

